@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2013-2018 Hannu Väisänen (Hannu.Vaisanen@uef.fi)
-# Program to generate old spellings and common spelling mistakes for Voikko lexicon.
+# Copyright 2013-2018, 2020 Hannu Väisänen
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +17,7 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 
+# Program to generate old spellings and common spelling mistakes for Voikko lexicon.
 # This automatic generation will generate some old
 # spellings and spelling errors that do not exist in real life.
 
@@ -34,25 +34,56 @@ from types import *
 sys.path.append("common")
 import generate_lex_common
 
+# Tulosta sanojen historiallinen taivutus.
+# Mallia on otettu funktiosta handle_word(word) tiedostossa generate_lex.py.
+#
+import voikkoutils
+import xml.dom.minidom
+def print_historical_inflection (word):
+    sukija_infclass = None
+    for infclass in word.getElementsByTagName ("infclass"):
+        if infclass.getAttribute("type") == "historical":
+            sukija_infclass = generate_lex_common.tValue (infclass)
+            break
+    if sukija_infclass is not None:
+        altforms = generate_lex_common.tValues (word.getElementsByTagName("forms")[0], "form")
+        for altform in altforms:
+            print ("SANA", altform, sukija_infclass)
+#voikkoutils.process_wordlist (generate_lex_common.VOCABULARY_DATA + '/joukahainen.xml', \
+#                              print_historical_inflection, False)
+#sys.exit(1)
+
+
 OPTIONS = generate_lex_common.get_options()
 
 infile = codecs.open (OPTIONS["destdir"] + u"/all.lexc", "r", "UTF-8")
 outfile = codecs.open (OPTIONS["destdir"] + u"/all-sukija.lexc", 'w', 'UTF-8')
 sukijafile = codecs.open (OPTIONS["destdir"] + u"/poikkeavat-sukija.lexc", 'r', 'UTF-8')
 
-C = u"[qwrtpsšdfghjklzžxcvbnm]"      # Kerakkeet.
-K = u"[qwrtpsšdfghjklzžxcvbnmaiou]"  # Kerakkeet + ääntiöitä.
-V = u"[aeiouüyåäö]"                  # Ääntiöt.
-A = u"[aä]"
-U = u"[uy]"
+C = "[qwrtpsšdfghjklzžxcvbnm]"      # Kerakkeet.
+M = "[bcdfghklmnpqrsštvwxzži]"      # Kerakkeet - j + i.
+J = "[bcdfghklmnpqrsštvwxzž]"       # Kerakkeet - j.
+K = "[qwrtpsšdfghjklzžxcvbnmaiou]"  # Kerakkeet + ääntiöitä.
+V = "[aeiouüyåäö]"                  # Ääntiöt.
+A = "[aä]"
+E = "[ei]"
+O = "[oö]"
+U = "[uy]"
+
 
 def makeRePattern (wordClass, word):
-    u = u"^\\[%s\\](\\[I..\\])?\\[Xp\\].*%s\\[X\\]" % (wordClass, word)
+    u = "^\\[%s\\](\\[I..\\])?\\[Xp\\].*%s\\[X\\]" % (wordClass, word)
+#    outfile.write ("!===== " + u + " " + word + "\n")
     u = u.replace ('C', C)
+    u = u.replace ('J', J)
     u = u.replace ('K', K)
+    u = u.replace ('M', M)
     u = u.replace ('V', V)
     u = u.replace ('A', A)
+    u = u.replace ('E', E)
+    u = u.replace ('O', O)
     u = u.replace ('U', U)
+#    outfile.write ("!===== " + u + "\n\n")
     return u
 
 
@@ -72,10 +103,54 @@ def replace_and_write (line, string1, string2):
     s = replace (line, string1, string2)
     outfile.write (s)
 
+# Spor(a)adi, spor(a)adinen, jne.
+#
+re_aCi    = makeRe ("(Ln|Ln|Lnl|Le.)", "(.{2,}C)(a)([df])(i)")
+re_aCia   = makeRe ("(Ln|Ln|Lnl|Le.)", "(.{2,}C)(a)([df]i)(a)")
+re_aCinen = makeRe ("(Ll|Ln|Lnl|Le.)", "(.{3,})(a)([df]i)(nen)")
 
-def replace_and_write_2 (line, string1, string2):
-    s = replace (line, string1, string2)
-    outfile.write (s)
+
+# -omi, -oni, -ori jne -loppuiset sanat; myös -omia, -ominen, jne.
+re_oMi    = makeRe ("(Ll|Ln|Lnl|Le.)", "(.{1,}M)(o)([dfglmnr])(i)")
+re_oMia   = makeRe ("(Ll|Ln|Lnl|Le.)", "(.{1,}M)(o)([dfglmnr])(ia)")
+re_oMinen = makeRe ("(Ll|Ln|Lnl|Le.)", "(.{1,}M)(o)([dfglmnr]i)(nen)")
+
+re_aali    = makeRe ("(Ll|Ln|Lnl)", "(.{2,})(aa)(l)(i)")
+re_aalinen = makeRe ("(Ll|Ln|Lnl)", "(.{2,})(aa)(li)(nen)")
+
+re_iivi    = makeRe ("(Ll|Ln|Lnl)", "(.{2,})(ii)(v)(i)")
+re_iivinen = makeRe ("(Ll|Ln|Lnl)", "(.{2,})(ii)(vi)(nen)")
+
+re_uuri = makeRe ("(Ll|Ln|Lnl)", "(.{1,}J)(uu)(r)(i)") # Ei muuteta juuri-loppuisia yhdyssanoja.
+re_yyri = makeRe ("(Ll|Ln|Lnl)", "(.{1,}C)(yy)(r)(i)")
+re_ööri = makeRe ("(Ll|Ln|Lnl)", "(.{1,}C)(öö)(r)(i)")
+
+re_torio = makeRe ("(Ll|Ln|Lnl)", "(.{1,}t)(o)(rio)")
+
+def find (line, list):
+    for u in list:
+#        outfile.write ("!!!! " + u + " " + list)
+        if line.find(u) >= 0:
+            return True
+    return False
+
+
+STOP = re.compile ("( |\t|@)")
+
+def end (line):
+    n = line.find(":")
+    r = STOP.search(line[n+1:])
+    return (n, n+r.start(1)+2)
+
+
+def ga (line, x):
+    if not find (line, x[0]):
+        e = end (line)
+        p = line[e[0]:e[1]]
+        q = p[:-x[1]-1] + x[2] + p[-x[3]-1:]
+#        if line.find("bromi") > 0:
+#            outfile.write ("!!HUUHAA [" + p + "]  [" + q + "]\n")
+        outfile.write (line.replace (p, q))
 
 
 re_oittaa1 = makeRe (u"Lt", u".Koittaa")
@@ -103,9 +178,6 @@ re_rangaista = re.compile (u"\\[Lt\\].* Rangaista_", re.UNICODE)
 
 re_Xiljoona = re.compile (u"\\A(?:\\[Bc\\]|\\[Sn\\]|@).*(b|m|tr)iljoon", re.UNICODE)
 
-re_tautua1 = re.compile ("\\[Lt\\].*tautua.*Kaatua_", re.UNICODE)
-re_tautua2 = re.compile ("\\[Lt\\].*täytyä.*Kaatua_", re.UNICODE)
-
 re_isoida_x = re.compile (u"\\A\[Lt\]\[Xp\](dramatisoida|karakterisoida)\[X\]")
 
 re_A = re.compile (u"[aou]")
@@ -115,8 +187,8 @@ re_oite  = makeRe (u"Ln", u".Coite")
 
 re_aatio = makeRe (u"Ln", u"..aatio")
 re_uutio = makeRe (u"Ln", u".Cuutio")
-re_uusio = makeRe (u"Ln", u".Cuusio")
 re_tio   = makeRe (u"Ln", u"([^a]i|k|p)tio") # Traditio, funktio, mutta ei aitio.
+re_uusio = makeRe (u"Ln", u"Cuusio")         # Myös fuusio.
 
 re_mmainen = re.compile (".*mmainen\\[X\\]")
 re_mmäinen = re.compile (".*mmäinen\\[X\\]")
@@ -139,11 +211,29 @@ def g6 (line, x):
         outfile.write (s)
 
 
+def replace4 (s, old, new, u):
+    return s.replace (old + u, new + u)
+
+
+def replace5 (s, old, new, u, v):
+    p = replace4 (s, old, new, u)
+    return replace4 (p, old, new, v)
+
+
 def gn (line, x):
-    for u in x[1]:
-        replace_and_write_2 (line, x[0], u)
-    for u in x[2]:
-        replace_and_write_2 (line.replace(x[3][0],x[3][1]), x[0], u)
+    if not line.startswith("[Ln][Xp]kuusio[X]"):
+        for u in x[1]:
+            s = replace5 (line, x[0], u, " ", "\t")
+            outfile.write (s)
+        s = line.replace (x[0] + "[X]", x[2][0] + "i[X]")
+        s = s.replace (x[3][0], x[3][1])
+        s = replace (s, x[0], x[2][0])
+        outfile.write (s)
+        s = replace5 (s, x[2][0], x[2][1], " ", "\t")
+        outfile.write (s)
+        s = line.replace (x[3][0], x[3][1])
+        replace_and_write (s, x[0], x[2][0])
+        replace_and_write (s, x[0], x[2][1])
 
 
 def g10 (line, x):
@@ -152,13 +242,36 @@ def g10 (line, x):
         replace_and_write (s, x[2], x[3])
 
 
+list_oMi    = ["blogi", "kromi", "aasiori", "hevosori", "jalostusori", "siitosori"]
+list_oMia   = ["victoria", "viktoria"]
+list_oMinen = ["hiusten=halkominen"]
+list_aali   = ["kraali", "shaali"]
+
 list_all = [
     (re_nuolaista, g2, ("Nuolaista_", "SukijaNuolaista_")),
     (re_rangaista, g2, ("Rangaista_", "SukijaRangaista_")),
-    (re_tautua1,   g2, ("Kaatua_",    "SukijaAntautua_")),
-    (re_tautua2,   g2, ("Kaatua_",    "SukijaAntautua_")),
     (re_isoida,   g10, ("isoida", "iseerata", "iso", "iseer", "Kanavoida", "Saneerata", "Voida", "Saneerata", re_isoida_x)),
 
+    (re_aCi,       ga, ([], 1, "a", 1)),
+    (re_aCia,      ga, ([], 2, "a", 2)),
+    (re_aCinen,    ga, ([], 2, "a", 2)),
+
+    (re_oMi,       ga, (list_oMi,    1, "o", 1)),
+    (re_oMia,      ga, (list_oMia,   2, "o", 2)),
+    (re_oMinen,    ga, (list_oMinen, 2, "o", 2)),
+
+#    (re_aali,      ga, (list_aali, 2, "", 1)),
+#    (re_aalinen,   ga, ([],        3, "", 2)),
+
+#    (re_iivi,      ga, (["aktiivi"], 2, "", 1)),
+#    (re_iivinen,   ga, ([], 3, "", 2)),
+
+#    (re_uuri,      ga, (["kulttuuri"], 2, "", 1)),
+#    (re_yyri,      ga, ([], 2, "", 1)),
+#    (re_ööri,      ga, ([], 2, "", 1)),
+
+#    (re_torio,     ga, ([], 3, "o", 3)),
+
     (re_oitin,     g6, ("oit:", "oit:", "oit ", "ot ",  "Suodatin",   "Suodatin")),
     (re_oite,      g6, ("oit:", "oit:", "oit ", "ot ",  "Vaate",      "Vaate")),
     (re_oittaa1,   g6, ("o:",   "oit:", "o ",   "ot ",  "Kirjoittaa", "Alittaa")),
@@ -178,8 +291,8 @@ list_all = [
     (re_oittua1,   g6, ("oit:", "oit:", "oit ", "ot ",  "Asettua",    "Asettua")),
     (re_oittua2,   g6, ("öit:", "öit:", "öit ", "öt ",  "Asettua",    "Asettua")),
 
-    (re_uusio,     gn, ("uusio", ("usio",),         ("usion",  "usioon"),  ("NimisanaAutio_a", "NimisanaPaperi_a"))),
-    (re_tio,       gn, ("tio",   ("tsio",),         ("tsion",  "tsioon"),  ("NimisanaAutio_a", "NimisanaPaperi_a"))),
+    (re_uusio,     gn, ("uusio", ("usio", ),        ("usion",  "usioon"),  ("NimisanaAutio_a", "NimisanaPaperi_a"))),
+    (re_tio,       gn, ("tio",   ("tsio", ),        ("tsion",  "tsioon"),  ("NimisanaAutio_a", "NimisanaPaperi_a"))),
     (re_aatio,     gn, ("aatio", ("atio", "atsio"), ("atsion", "atsioon"), ("NimisanaAutio_a", "NimisanaPaperi_a"))),
     (re_uutio,     gn, ("uutio", ("utio", "utsio"), ("utsion", "utsioon"), ("NimisanaAutio_a", "NimisanaPaperi_a"))),
 
@@ -199,7 +312,8 @@ def word_class (line):
     L = dict ([("[Lep]", "Paikannimi"),
                ("[Ll]",  "Laatusana"),
                ("[Ln]",  "Nimisana"),
-               ("[Lnl]", "NimiLaatusana")])
+               ("[Lnl]", "NimiLaatusana"),
+               ("[Lt]",  "")])
     return L[line[0:line.find("]")+1]]
 
 
@@ -231,6 +345,14 @@ def write_paahtaa (line, word):
     prefix = line[0:line.find (u" ")]
     outfile.write (u"%s SukijaPaahtaa_a ;\n" % (prefix))
 
+def write_taittaa (line, word):
+    prefix = line[0:line.find (u" ")]
+    outfile.write (u"%s SukijaTaittaa_a ;\n" % (prefix))
+
+def write_palata (line, word):
+    prefix = line[0:line.find (u" ")]
+    outfile.write (u"%s SukijaPalata_a ;\n" % (prefix))
+
 
 word_list = [
     (u"tällainen",       ((u"tällai",      u"tällai", u"NimiLaatusanaNainenInen_a", u"NimiLaatusanaNainenInen_ä"),
@@ -337,7 +459,7 @@ function_list = [
     (lambda line, word: outfile.write (u"[Ln][Xp]%s[X]%s:%s SukijaLohi ;\n" %
                                        (word, word[0:len(word)-1], word[0:len(word)-1])),
      (u"lohi",
-      u"tyynenmerenlohi",   # On Joukahaisessa.
+      u"tyynen=meren=lohi",   # On Joukahaisessa.
       u"uuhi")),
 
     # 34 lahti (2, 2). Tuomi, s. 193.
@@ -390,16 +512,25 @@ function_list = [
 
      # 11 paistaa (9, 9). Tuomi s. 1, 2, 8, 11, 12, 15, 17.
      #
-     (write_virkkaa, (u"vilkkaa",
-                      u"virkkaa")),
-     (write_paistaa, (u"paistaa",
-                      u"uppo=paistaa")),
-     (write_paahtaa, (u"paahtaa",
-                      u"raistaa",
-                      u"saattaa",
-                      u"taittaa",
-                      u"palttaa",
-                      u"varttaa")),
+     (write_virkkaa, ("vilkkaa",
+                      "virkkaa")),
+     (write_paistaa, ("maustaa",  # Ei ole Tuomella, mutta Lönnrot-projektin kirjoissa taivutetaan maustin, maustit, maustimme...
+                      "paistaa",
+                      "raistaa",
+                      "uppo=paistaa")),
+     (write_paahtaa, ("kaihtaa",
+                      "paahtaa", )),
+     (write_taittaa, ("saattaa",
+                      "taittaa",
+                      "palttaa",
+                      "varttaa")),
+
+     # Avata -> avaja jne. Nykysuomen sanakirjan teonsanojen taivutuskaava 40.
+     (write_palata,  ("avata",
+                      "halata",
+                      "palata")),
+     (lambda line, word: write_word (line, word, "SukijaLevätä"), ("levätä", )),
+     (lambda line, word: write_word (line, word, "SukijaHylätä"), ("hyljätä", "hylätä")),
 ]
 
 def convert_to_dictionary (word_list):
@@ -491,10 +622,40 @@ ei_vertm = re.compile (u"@[PDC][.]EI_VERTM([.]ON)?@", re.UNICODE)
 ei_yks = "@P.EI_YKS.ON@"
 
 
+# Yksikintoista, yksikääntoista, kaksikaantoista, jne.
+#
+LUKUSANA_TOISTA =\
+  "[Bc]toista[Xp][X]:kintoista	Liitesana_a	;\n" +\
+  "[Bc]toista:kintoista	LukusananJälkiliite	;\n" +\
+  "[Bc]toista:kintoista	LukusanaKerroinTuhatta	;\n" +\
+  "[Bc]toista:kintoista	LukusananAlkuKerroin	;\n" +\
+  "[Bc]toista[Xp][X]:kaantoista	Liitesana_a	;\n" +\
+  "[Bc]toista:kaantoista	LukusananJälkiliite	;\n" +\
+  "[Bc]toista:kaantoista	LukusanaKerroinTuhatta	;\n" +\
+  "[Bc]toista:kaantoista	LukusananAlkuKerroin	;\n" +\
+  "[Bc]toista[Xp][X]:kääntoista	Liitesana_a	;\n" +\
+  "[Bc]toista:kääntoista	LukusananJälkiliite	;\n" +\
+  "[Bc]toista:kääntoista	LukusanaKerroinTuhatta	;\n" +\
+  "[Bc]toista:kääntoista	LukusananAlkuKerroin	;\n"
+
+
+# Yhdeskintoista jne.
+#
+JÄRJESTYSLUKU_TOISTA = \
+  "[Bc]toista[Xp][X]:kintoista	Liitesana_a	;\n" +\
+  "[Bc]toista:kintoista	LukusananJälkiliiteJl	;\n" +\
+  "[Bc]toista[Xp][X]:kaantoista	Liitesana_a	;\n" +\
+  "[Bc]toista:kaantoista	LukusananJälkiliiteJl	;\n" +\
+  "[Bc]toista[Xp][X]:kääntoista	Liitesana_a	;\n" +\
+  "[Bc]toista:kääntoista	LukusananJälkiliiteJl	;\n"
+
+
 sukija_additions = {
     "LEXICON Asemosana\n":     "SukijaAsemosana ;\n",
+    "LEXICON Sanasto_ee\n":    "SukijaPoikkeavat_ee ;\n",
     "LEXICON Sanasto_em\n":    "SukijaPoikkeavat_em ;\n",
     "LEXICON Sanasto_ep\n":    "SukijaPoikkeavat_ep ;\n",
+    "LEXICON Sanasto_h\n":     "SukijaPoikkeavat_h ;\n",
     "LEXICON Sanasto_l\n":     "SukijaPoikkeavat_l ;\n",
     "LEXICON Sanasto_n\n":     "SukijaPoikkeavat_n ;\n",
     "LEXICON Sanasto_nl\n":    "SukijaPoikkeavat_nl ;\n",
@@ -503,7 +664,10 @@ sukija_additions = {
     "LEXICON Sanasto_s\n":     "SukijaPoikkeavat_s ;\n",
     "LEXICON Suhdesana\n":     "SukijaSuhdesana ;\n",
     "LEXICON LukusananErikoisjälkiliite\n": "SukijaLukusananErikoisjälkiliite ;\n",
-    "LEXICON Lukusana\n":        "SukijaLukusanaSeitsemän ;\n",
+    "LEXICON Lukusana\n":            "SukijaLukusana ;\n",
+    "LEXICON Järjestysluku\n":       "SukijaJärjestysluku ;\n",
+    "LEXICON LukusanaToista\n":      LUKUSANA_TOISTA, 
+    "LEXICON JärjestyslukuToista\n": JÄRJESTYSLUKU_TOISTA,
     "LEXICON Omistusliite_a\n":  "[O2y]s:s Liitesana_a  ;\n",
     "LEXICON Omistusliite_ä\n":  "[O2y]s:s Liitesana_ä  ;\n",
     "LEXICON Omistusliite_aä\n": "[O2y]s:s Liitesana_aä ;\n"
@@ -543,11 +707,10 @@ while True:
         line = line.replace (u"@P.YS_EI_JATKOA.ON@", u"")
         line = line.replace (u"@D.YS_EI_JATKOA@", u"")
         line = line.replace (u"@C.YS_EI_JATKOA@", u"")
+
     outfile.write (line)
     write_sukija_additions (line, sukija_additions)
-
     generate_all (line, list_all)
-
     write_arvaella (line)
 
     r = base_form_re.search (line)
